<?php
/*

 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 
 
 */

/**
 * <p>Renders Cash on Delivery description text to order confirmation and e-mail templates</p>
 * <p>Description is only displayed when merchant sets description for DPD cash on delivery method</p>
 *
 * @author Matis
 */
class Eabi_DpdEE_Block_Info_Payment extends Mage_Payment_Block_Info_Cc {
    
    const INFO_TEXT = 'info_text';


    protected $_paymentInfoMap = array(
        self::INFO_TEXT => 'info_text',
    );
    protected $_customerInfoMap = array(
        self::INFO_TEXT => 'info_text',
    );

    public function getCcTypeName() {
        
    }

    /**
     * <p>Returns <code>info_text</code> field, when merchant decides to display description in cash on delivery method</p>
     * @param array $transport
     * @return array
     */
    protected function _prepareSpecificInformation($transport = null) {
        $transport = parent::_prepareSpecificInformation($transport);
        /* @var $payment Mage_Payment_Model_Info */
        $payment = $this->getInfo();
        $info = array();
        if ($this->getIsSecureMode()) {
            foreach ($this->_customerInfoMap as $key) {
                if (!isset($info[$key])) {
                    $info[$key] = array();
                }
                if (!$payment->hasAdditionalInformation($key)) {
//                    $info[$key]['label'] = false;
//                    $info[$key]['value'] = false;
                } else {
                    $value = $payment->getAdditionalInformation($key);
                    if ($value) {
                        $info[$key]['label'] = $this->_getLabel($key);
                        $info[$key]['value'] = $value;
                    }
                }
            }
        } else {
            foreach ($this->_paymentInfoMap as $key) {
                if (!isset($info[$key])) {
                    $info[$key] = array();
                }
                if (!$payment->hasAdditionalInformation($key)) {
                    $info[$key]['label'] = $this->_getLabel($key);
                    $info[$key]['value'] = false;
                } else {
                    $value = $payment->getAdditionalInformation($key);
                    if ($value) {
                        $info[$key]['label'] = $this->_getLabel($key);
                        $info[$key]['value'] = $value;
                    }
                }
            }
        }
        $resultingInfo = array();
        foreach ($info as $value) {
            if (isset($value['value']) && trim($value['value']) && isset($value['label'])) {
                $resultingInfo[$value['label']] = $value['value'];
            }
        }
        if (count($resultingInfo)) {
            return $transport->addData($resultingInfo);
        }
        return $transport;

    }

    /**
     * 
     * @param string $key
     * @return string
     */
    protected function _getLabel($key) {
        switch ($key) {
            case self::INFO_TEXT:
                return $this->_getDpdHelper()->__('Instructions');
                break;
        }
        return '';
    }
    
    /**
     * 
     * @return Eabi_DpdEE_Helper_Data
     */
    protected function _getDpdHelper() {
        return Mage::helper('eabi_dpdee');
    }
    

}

