<?php

/*
  
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * <p>DPD Cash on delivery payment method</p>
 * <p>Displays optional description and additional cash on delivery fee when set.</p>
 * <p>Cash on delivery fee would be added on top of shipping price</p>
 *
 * @author matishalmann
 */
class Eabi_DpdEE_Model_Payment_Processor extends Mage_Payment_Model_Method_Abstract {

    protected $_code = Eabi_DpdEE_Model_Config::PAYMENT_METHOD_COD;
    protected $_canAuthorize = true;
    protected $_canCapture = true;
    protected $_canVoid = true;
    protected $_canUseInternal = true;
    protected $_canUseCheckout = true;
    protected $_canUseForMultishipping = true;
    protected $_formBlockType = 'eabi_dpdee/invoice';
    protected $_infoBlockType = 'eabi_dpdee/info_payment';
    protected $_checkout;
    protected $_quote;

    /**
     *
     * @var Eabi_DpdEE_Model_Post
     */
    private static $_methodInstance;
    /**
     *
     * @var string
     */
    private $_currency;
    
    
    /**
     * <p>Assigns custom order status</p>
     * @param string $action
     * @param Mage_Sales_Model_Order $stateObject
     * @return Eabi_DpdEE_Model_Payment_Processor
     */
    public function initialize($action, $stateObject) {
        if(($status = $this->getConfigData('order_status'))) {
            $stateObject->setStatus($status);
            $state = $this->_getAssignedState($status);
            $stateObject->setState($state);
            $stateObject->setIsNotified(true);
        }
        return $this;
    }    
    
    
    

    /**
     * <p>This method is available when:</p>
     * <ul>
         <li>Payment method is active</li>
         <li>Order shipping address is destined to country which has cash on delivery price defined and not left empty</li>
         <li>Cart contains only products which are in stock if in stock setting is enabled</li>
     </ul>
     * @param Mage_Sales_Model_Quote $quote
     * @return boolean
     */
    public function isAvailable($quote = null) {
        
        $ret = parent::isAvailable($quote);
        if (!$ret) {
            self::$_methodInstance = null;
            return false;
        }
        if ($quote === null || !$quote->getShippingAddress() || !$quote->getShippingAddress()->getShippingMethod()) {
            self::$_methodInstance = null;
            return false;
        }
        $method = $quote->getShippingAddress()->getShippingMethod();
        //get the method instance....
        
        $shippingMethodModel = Mage::getModel('shipping/shipping')->getCarrierByCode(substr($method, 0, strpos($method, '_')));
        if ($shippingMethodModel && ($shippingMethodModel instanceof Eabi_DpdEE_Model_Post) && $shippingMethodModel->isCodEnabled($quote->getShippingAddress())) {
            
            if ($this->getConfigData('checkstore')) {
                if ($quote->getAllItems()) {
                    foreach ($quote->getAllItems() as $item) {
                        if (!$item->getProduct()->isVirtual()) {
                            if ($item->getProduct()->getStockItem()->getQty() < 1) {
                                self::$_methodInstance = null;
                                return false;
                            }
                        }
                    }
                }
            }
            self::$_methodInstance = $shippingMethodModel;
        } else {
            self::$_methodInstance = null;
            return false;
        }
        return true;
    }

    /**
     * <p>Internal payment authorization method</p>
     *
     * @param   Varien_Object $orderPayment
     * @return  Mage_Payment_Model_Abstract
     */
    public function authorize(Varien_Object $payment, $amount) {
        //    	print_r($payment);
        parent::authorize($payment, $amount);
        return $this;
    }

    /**
     * <p>Internal payment capture method</p>
     *
     * @param   Varien_Object $orderPayment
     * @return  Mage_Payment_Model_Abstract
     */
    public function capture(Varien_Object $payment, $amount) {

        return $this;
    }

    /**
     * <p>Internal payment void method</p>
     *
     * @param   Varien_Object $invoicePayment
     * @return  Mage_Payment_Model_Abstract
     */
    public function void(Varien_Object $payment) {
        return $this;
    }

    /**
     * <p>Fetches current currency to this object and then checks it validity</p>
     * <p>Internal method only checks for validity for this currency</p>
     * @param string $currencyCode
     * @return bool
     */
    public function canUseForCurrency($currencyCode) {
        $this->_currency = $currencyCode;
        return parent::canUseForCurrency($currencyCode);
    }

    /**
     * <p>Displays current payment method title with extra cash on delivery fee when extra fee has been set</p>
     * @return string
     */
    public function getTitle() {
        
        if (self::$_methodInstance && $this->getQuote()->getShippingAddress() && self::$_methodInstance->getCodFee($this->getQuote()->getShippingAddress()) > 0) {
            $rawFee = (float) self::$_methodInstance->getCodFee($this->getQuote()->getShippingAddress());
            if (!$this->_currency) {
                $this->_currency = Mage::app()->getStore()->getCurrentCurrencyCode();
            }
            $baseCurrency = Mage::app()->getStore()->getBaseCurrencyCode();
            //convert to currency
            $price = $this->getShippingPrice(Mage::helper('directory')->currencyConvert($rawFee, $baseCurrency, $this->_currency), null);

            $descriptionText = $this->_getDpdHelper()->__('Will be added to shipping fee');
            if (self::$_methodInstance->getData('cod_fee_added')) {
                $descriptionText = $this->_getDpdHelper()->__('Already added to shipping fee');
            }
            return parent::getTitle() . ' (' . Mage::helper('core')->currency($price, true, false) . ' ' . $descriptionText . ')';
        } else {
            return parent::getTitle();
        }
    }

    /**
     * <p>Returns final shipping price depending on shipping address and tax settings</p>
     * @param double $price
     * @param bool $flag
     * @return double
     */
    public function getShippingPrice($price, $flag) {
        //takes into account whether shipping price is //tax/calculation/shipping_includes_tax or not
        return Mage::helper('tax')->getShippingPrice($price, $flag, $this->getQuote()->getShippingAddress());
    }

    /**
     * <p>Assigns cash on delivery payment method description to payment method blocks so they would be displayed in e-mails and order confirmations</p>
     *
     * @param   mixed $data
     * @return  Eabi_Cod_Model_Payment
     */
    public function assignData($data) {
        $details = array();
        if ($data instanceof Varien_Object) {
            $data = $data->getData();
        }
        if ($this->getInfoText()) {
            $details['info_text'] = $this->getInfoText();
        }
        if (count($details)) {
            $this->getInfoInstance()->setAdditionalData(serialize($details));
            foreach ($details as $k => $v) {
                $this->getInfoInstance()->setAdditionalInformation($k, $v);
            }
        }
        
        return $this;
    }
    
    
    /**
     * <p>Fetches order status name by code</p>
     * @param string $status
     * @return string
     */
    protected function _getAssignedState($status) {
        $item = Mage::getResourceModel('sales/order_status_collection')
            ->joinStates()
            ->addFieldToFilter('main_table.status', $status)
            ->getFirstItem();
 
        return $item->getState();
    }
    
    
    /**
     * 
     * @return Mage_Sales_Model_Quote_Item
     */
    protected function _getQuoteItemModel() {
        return Mage::getModel('sales/quote_item');
    }
    
    
    /**
     * 
     * @return Mage_Catalog_Model_Product
     */
    protected function _getProductModel() {
        return Mage::getModel('catalog/product');
    }

    /**
     * <p>Returns current methods description</p>
     * @return string
     */
    public function getInfoText() {
        return $this->getConfigData('info_text');
    }

    /**
     * 
     * @return Eabi_DpdEE_Helper_Data
     */
    protected function _getDpdHelper() {
        return Mage::helper('eabi_dpdee');
    }

    /**
     * <p>Retrieves current checkout session model</p>
     *
     * @return Mage_Checkout_Model_Session
     */
    public function getCheckout() {
        if (empty($this->_checkout)) {
            if (Mage::app()->getStore()->isAdmin()) {
                $this->_checkout = Mage::getSingleton('adminhtml/session_quote');
            } else {
                $this->_checkout = Mage::getSingleton('checkout/session');
            }
        }
        return $this->_checkout;
    }

    /**
     * Retrieve sales quote model
     *
     * @return Mage_Sales_Model_Quote
     */
    public function getQuote() {
        if (empty($this->_quote)) {
            $this->_quote = $this->getCheckout()->getQuote();
        }
        return $this->_quote;
    }

}
