<?php

/*
   *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Livehandler
 * @copyright  Copyright (c) 2018 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * Description of Licence
 *
 * @author Matis
 * @method Eabi_Livehandler_Block_Licence setFormFieldId(string $formFieldId)
 * @method string getFormFieldId()
 * @method Eabi_Livehandler_Block_Licence setFormFieldName(string $formFieldName)
 * @method string getFormFieldName()
 * @method Varien_Data_Form_Element_Abstract getElement()
 * @method Eabi_Livehandler_Block_Licence setElement(Varien_Data_Form_Element_Abstract $element)
 */
class Eabi_Livehandler_Block_Licence extends Mage_Core_Block_Template {
    
    public static $certificate = '-----BEGIN CERTIFICATE REQUEST-----
MIIB3TCCAUYCAQAwgZwxCzAJBgNVBAYTAkVFMREwDwYDVQQIEwhIYXJqdW1hYTEQ
MA4GA1UEBxMHVGFsbGlubjEbMBkGA1UECgwSQWt0c2lhbWFhaWxtIE/Dg8KcMRUw
EwYDVQQLEwx3d3cuZS1hYmkuZWUxFjAUBgNVBAMTDW1hdGlzIGhhbG1hbm4xHDAa
BgkqhkiG9w0BCQEWDWluZm9AZS1hYmkuZWUwgZ8wDQYJKoZIhvcNAQEBBQADgY0A
MIGJAoGBAOaMZSq2d0u1G2mZVoJrFNhVwY5+srRvIPfdjN/fb9r6akHah4sgO/Me
NavqsqSaIyaVHk+qsmoobME5xLCfqfpqLVZUWWl1Gl/y/iDw51sQoZVtPJUBcNaE
L1HR4MU+7b5/Ig+IpCdPpKZ0xxTIFa731B4jzscIZQZYuljrn3xDAgMBAAGgADAN
BgkqhkiG9w0BAQUFAAOBgQAf+bJJ+j+4kPE+Ih5BLXGeDvc4P8UxvEjCLrzFhysD
z613BUu93jwLSTuZwHcbUtDyGgUAhWAtDmVZZk1ZksR1LZ72qV6agj0brx+O4M22
p8HWZm6dNmAtVUJk4fuqlPdkwKOfb4Fju8NhfeiPqOk363Z2oSdUkityaHF0/OJa
RA==
-----END CERTIFICATE REQUEST-----';
    private static $_publicKey;
    protected $blockValue;

    /**
     *
     * @var Eabi_Livehandler_Model_Licenceable
     */
    protected $_method = array();
    protected $_methods = array();
    protected $_columns = array();
    private static $_callOuts = array();
    
    
    public function _construct() {
        $this->setTemplate('eabi_livehandler/licence.phtml');
        if (!self::$_publicKey) {
            self::$_publicKey = openssl_csr_get_public_key(self::$certificate);
        }
        
        parent::_construct();
    }


    public function getLogo() {
        $html = '';
        $title = htmlspecialchars(sprintf($this->_getEabi()->__('Support from e-mail %s'), 'info@e-abi.ee'));
        if ($this->getMethod()) {
            $html .= sprintf('<img alt="' . $title . '" title="' . $title . '" src="http://www.e-abi.ee/skin/frontend/default/electronics3/images/%s" style="height: 30px; width: 30px;"/>', $this->getMethod()->getConfigData('eabi_logo') ? $this->getMethod()->getConfigData('eabi_logo'): 'logo-web.png');
        } else {
            $html .= '<img alt="' . $title . '" title="' . $title . '" src="http://www.e-abi.ee/skin/frontend/default/electronics3/images/logo-web.png" style="height: 30px; width: 30px;"/>';
        }
        return $html;
    }

    public function setMethod($method) {
        if (is_string($method)) {
            $this->_methods = array($method);
            $method = $this->getMethodByCode($method);
        }
        $this->_method = $method;
        return $this;
    }

    public function setMethods(array $methods) {
        $this->_methods = $methods;
        return $this;
    }

    public function getMethods() {
        if (!count($this->_methods)) {
            return array($this->getMethod()->getCode());
        }
        return $this->_methods;
    }

    public function getMethodByCode($code) {
        //Mage::getModel('shipping/shipping')->getCarrierByCode($shippingCarrierCode, $order->getStoreId())
        $types = explode('/', $code);
        $allShippingMethods = $this->_getEabi()->getAllSupportedMethodCodes();
        $method = $allShippingMethods->getItemById($code);

        if (!is_object($method)) {
            return null;
        }

        return $method->getMethodInstance();
    }
    public function getRawMethodByCode($code) {
        //Mage::getModel('shipping/shipping')->getCarrierByCode($shippingCarrierCode, $order->getStoreId())
        $types = explode('/', $code);
        $allShippingMethods = $this->_getEabi()->getAllSupportedMethodCodes();
        return $allShippingMethods->getItemById($code);
    }

    /**
     * 
     * @return Eabi_Livehandler_Model_Licenceable
     */
    public function getMethod() {
        return $this->_method;
    }

    public function setValue($value) {
        $this->blockValue = $value;
        return $this;
    }

    public function getValue() {
        return $this->blockValue;
    }
    
    protected function _removeExpiredEntries($licenceDatas) {
        $finalDatas = array();
        if (!is_array($licenceDatas)) {
            return $finalDatas;
        }
        foreach ($licenceDatas as $service => $licenceData) {
            $serviceDatas = $this->_removeExpiredEntriesOnService($licenceData);
            if (count($serviceDatas)) {
                $finalDatas[$service] = $serviceDatas;
            }
        }
        return $finalDatas;
    }
    protected function _removeExpiredEntriesOnService($licenceDatas) {
        $entries = array();
        foreach ($licenceDatas as $licenceData) {
            
            if ($licenceData[$this->_getS()] >= time()) {
                $entries[] = $licenceData;
            }
        }
        return $entries;
    }

    public function getLicenceStatus($serviceCode, $country) {
        $text = sprintf('<span class="eabi-postoffice-red">%s</span>', $this->_getEabi()->__('Disabled'));
        $textChanged = false;
        $d = '';
        $u = $this->_getU();
        $v = $this->_getV();
        $q = $this->_getQ();
        $licenceData = array();


        if ($this->getValue() || in_array($serviceCode, $this->getMethods())) {

            if ($this->getValue()) {
                $licenceData = json_decode(@gzuncompress($v($this->getValue())), true);
            }

            if (!count($licenceData) || !isset($licenceData[$serviceCode . '_' . $country])) {
                if ($this->getMethod()->getCode() != $serviceCode) {
                    $shippingMethod = $this->getMethodByCode($serviceCode);
                    $licenceData = @json_decode(@gzuncompress($v($this->getRawMethodByCode($serviceCode)->getRawLicence())), true);

                }
            }
            $licenceData = $this->_removeExpiredEntries($licenceData);
            

            if (isset($licenceData[$serviceCode . '_' . $country])) {
                foreach ($licenceData[$serviceCode . '_' . $country] as $licenceArray) {

                    $licenceString = $this->_getLicenceString($licenceArray, $serviceCode, $country);
                    if ($u($licenceString, $v($licenceArray[$this->_getT()]), self::$_publicKey, $this->_getM())) {
                        if ($licenceArray[$this->_getO()]) {
                            $ed = new $q($licenceArray[$this->_getS()], Zend_Date::TIMESTAMP);
                            
                            $df = Mage::app()->getLocale()->getDateFormat(Mage_Core_Model_Locale::FORMAT_TYPE_SHORT);
                            $text = sprintf('<span class="eabi-postoffice-yellow">%s</span>', $this->_getEabi()->__('Demo'));
                            $text .= ' ' . sprintf($this->_getEabi()->__('(expires: %s)'), $ed->get($df));
                        } else {
                            $text = sprintf('<span class="eabi-postoffice-green">%s</span>', $this->_getEabi()->__('Activated'));
                        }
                        $textChanged = true;
                        break;
                    }
                }
                if (!$textChanged) {
                    $this->setValue("");
                    $this->reset();
                }
                
            }
        }
        return $text . $d;
    }
    
    

    public function isServiceAvailable($serviceCode, $country) {
        return $this->_isServiceAvailable($serviceCode, $country) || $this->_isServiceAvailable($serviceCode, '*');
    }

    protected function _getK() {
        return str_rot13('fge_ebg13');
    }

    protected function _getL() {
        $a = $this->_getK();
        return $a('pbafgnag');
    }

    protected function _getM() {
        $a = $this->_getL();
        $b = $this->_getK();
        return $a($b('BCRAFFY_NYTB_FUN1'));
    }

    protected function _getN() {
        $a = $this->_getK();
        return $a('unfu');
    }

    protected function _getO() {
        $a = $this->_getK();
        return $a('qrzbovg');
    }

    protected function _getP() {
        $a = $this->_getK();
        return $a('fnyg');
    }

    protected function _getQ() {
        $a = $this->_getK();
        return $a('Mraq_Qngr');
    }

    protected function _getS() {
        $a = $this->_getK();
        return $a('rkcvengvba_fgnzc');
    }

    protected function _getT() {
        $a = $this->_getK();
        return $a('fvtangher');
    }

    protected function _getU() {
        $a = $this->_getK();
        return $a('bcraffy_irevsl');
    }

    protected function _getV() {
        $a = $this->_getK();
        return $a('onfr64_qrpbqr');
    }

    protected function _getW() {
        $a = $this->_getK();
        return $a('qngr_sbezng');
    }

    protected function _getX() {
        $a = $this->_getK();
        return $a('yvprapr');
    }

    protected function _getY() {
        $a = $this->_getK();
        return $a('onfr64_rapbqr');
    }

    protected function _getZ() {
        $a = $this->_getK();
        return $a('fun256');
    }

    protected function _isServiceAvailable($serviceCode, $country) {
        $s = $this->_getS();
        $t = $this->_getT();
        $u = $this->_getU();
        $v = $this->_getV();
        $x = $this->_getX();
        

        $shippingMethod = $this->getMethodByCode($serviceCode);
        if (is_null($shippingMethod)) {
            return true;
        }
        $licenceData = @json_decode(@gzuncompress($v($this->getRawMethodByCode($serviceCode)->getRawLicence())), true);
        $result = false;
        
        if (isset($licenceData[$serviceCode . '_' . $country])) {
            
            foreach ($licenceData[$serviceCode . '_' . $country] as $licenceArray) {
                $licenceString = $this->_getLicenceString($licenceArray, $serviceCode, $country);
                if ($u($licenceString, $v($licenceArray[$t]), $this->getPublicKey(true), $this->_getM())) {

                    if ($licenceArray[$s] >= time()) {
                        $result = true;
                    } else {
                        $result = false;
                    }
                    break;
                }
            }
        }
        
        $this->_handleCallout($serviceCode, $country, $result);
        return $result;
    }
    
    

    public function getStatus($serviceCode, $country) {
        if (isset(self::$_callOuts[$serviceCode]) && isset(self::$_callOuts[$serviceCode][$country])) {
            return self::$_callOuts[$serviceCode][$country];
        }
        if (isset(self::$_callOuts[$serviceCode]) && isset(self::$_callOuts[$serviceCode]['*'])) {
            return self::$_callOuts[$serviceCode]['*'];
        }
        return null;
    }

    public function getAllSiteHosts() {
        $configPaths = array(
            'web/unsecure/base_url',
            'web/secure/base_url',
        );
        
        $configDatas = $this->_getEabi()
                ->getCoreConfigDataModel()
                ->getCollection()
                ->addFieldToFilter('path', array('in' => $configPaths));
        $hosts = array();
        foreach ($configDatas as $configData) {
            try {
                $host = Zend_Uri_Http::fromString($configData->getValue());
                if (!in_array($host->getHost(), $hosts)) {
                    $hosts[] = $host->getHost();
                }
            } catch (Zend_Uri_Exception $ex) {
                //do nothing

            }
            
        }
        return $hosts;
    }

    public function splitLicenceToServices($compressedLicenceData, &$isNew = false) {
        $v = $this->_getV();
        $y = $this->_getY();
        $licenceData = @json_decode(@gzuncompress($v($compressedLicenceData)), true);
        $isFinal = isset($licenceData['is_final']) && $licenceData['is_final'] === true;

        if ($isFinal) {
            $isNew = true;
            unset($licenceData['is_final']);
        }
        $allMethodCodes = $this->_getEabi()->getAllSupportedMethodCodes();
        $result = array();
        if (!$licenceData) {
            return $result;
        }
        foreach ($licenceData as $serviceCombination => $serviceDatas) {
            $serviceKey = substr($serviceCombination, 0, -3);
            $serviceCountry = substr($serviceCombination, -2);
            if ($serviceCountry == '_*') {
                $serviceKey = substr($serviceCombination, 0, -2);
                $serviceCountry = substr($serviceCombination, -1);
            }
            //
            $serviceKeyCombinations = explode('/', $serviceKey);
            
            if (!isset($result[$serviceKeyCombinations[1]])) {
                $result[$serviceKeyCombinations[1]] = array();
            }
//            $result[$serviceKey][$serviceCombination] = $serviceDatas;
            $result[$serviceKeyCombinations[1]][$serviceCombination] = $this->_validateAndClearLicenceData($serviceDatas);
        }
        //TODO: some sort of validation?
        foreach ($result as $serviceKey => $data) {
            $result[$serviceKey] = $y(gzcompress(json_encode($data)));
        }
        if ($isFinal) {
            foreach ($allMethodCodes as $methodCode) {
//                echo '<pre>'.htmlspecialchars(print_r($methodCode->debug(), true), ENT_COMPAT | ENT_HTML401 | ENT_IGNORE).'</pre>';
                
                if (!isset($result[$methodCode->getCode()])) {
                    $result[$methodCode->getCode()] = '';
                }
            }
        }
        return $result;
    }
    
    public function reset() {
        $shippingMethodCodes = $this->_getEabi()->getAllSupportedMethodCodes();
        foreach ($shippingMethodCodes as $shippingMethodCode) {
            try {
                $shippingMethod = $shippingMethodCode->setRawLicence('');
            } catch (Exception $ex) {
                $this->_getEabi()
                        ->getEabiLogger()
                        ->setIsLogEnabled(true)
                        ->setLogLevel(Zend_Log::DEBUG)
                        ->debug($ex->__toString());
            }
        }
        return $this;
    }
    

    protected function _validateAndClearLicenceData($datas) {
        $fields = array(
            $this->_getP() => array('length' => 64),
            $this->_getO() => array('length' => 1),
            $this->_getS() => array('length' => 19),
            $this->_getT() => array('length' => 256),
        );
        $result = array();
        foreach ($datas as $i => $data) {
            $result[$i] = array();
            foreach ($fields as $field => $descriptor) {
                if (!isset($data[$field]) || strlen($data[$field]) > $descriptor['length']) {
                    Mage::throwException(sprintf($this->_getEabi()->__('Field %s was probably manipulated'), $field));
                }
                $result[$i][$field] = $data[$field];
            }
        }
        return $result;
    }

    protected function _getLicenceString(array $licenceData, $serviceCode, $country) {
        $d = '';
        $n = $this->_getN();
        $d .= str_pad(parse_url(Mage::getUrl(), PHP_URL_HOST), 500, ' ', STR_PAD_RIGHT);
        $d .= str_pad($licenceData[$this->_getP()], 64, ' ', STR_PAD_RIGHT);
        $d .= str_pad($serviceCode, 255, ' ', STR_PAD_RIGHT);
        $d .= str_pad($country, 2, ' ', STR_PAD_RIGHT);
        $d .= str_pad($licenceData[$this->_getO()], 1, ' ', STR_PAD_RIGHT);

        //timeout
        $time = ltrim($licenceData[$this->_getS()], 0);


        if (time() > $time) {
            $d .= str_pad('0', 19, '0', STR_PAD_LEFT);
        } else {
            $d .= str_pad($licenceData[$this->_getS()], 19, '0', STR_PAD_LEFT);
        }


        return $n($this->_getZ(), $d);
    }

    public function getPublicKey($asResource = false) {
        if ($asResource) {
            return self::$_publicKey;
        } else {
            return self::$certificate;
        }
    }

    public function getCountryName($input) {
        $countries = $this->_getEabi()->toAssoc($this->_getEabi()->getEabiSupportedCountries()->toOptionArray());
        return $countries[$input];
    }

    public function getRegistrationUrl() {
        return Eabi_Livehandler_Model_Config::REGISTRATION_URL;
    }

    public function getRegisterButtonLabel() {
        return $this->_getEabi()->__('Register Licence');
    }

    public function getDeleteButtonLabel() {
        return $this->_getEabi()->__('Delete');
    }

    protected function _handleCallout($serviceCode, $country, $result = false) {
        if (!isset(self::$_callOuts[$serviceCode])) {
            self::$_callOuts[$serviceCode] = array();
        }
        $oldCallout = $serviceCode . '_' . $country;
        if (isset(self::$_callOuts[$serviceCode][$country])) {
            $oldCallout = self::$_callOuts[$serviceCode][$country];
        }
        if (!$result) {
            $oldCallout = false;
            unset(self::$_callOuts[$serviceCode][$country]);
            return;
        }
        
        self::$_callOuts[$serviceCode][$country] = md5($oldCallout);
    }
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabi() {
        return Mage::helper('eabi');
    }
    
}
