<?php

/*
   *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Livehandler
 * @copyright  Copyright (c) 2018 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * Description of Licence
 *
 * @author Matis
 */
class Eabi_Livehandler_Model_Backend_Licence extends Mage_Core_Model_Config_Data {
    protected static $_hasLicenceBeenSplit = false;
    protected $_wasLicenceSplit = false;
    protected function _beforeSave() {
        $this->setScopeId(0);
        $this->setScope('default');
        $this->setStoreCode(null);
        $this->setWebsiteCode(null);
        if ($this->isValueChanged()) {
            $value = $this->getValue();
            $isFinal = false;
            $splitResult = $this->alterLicenceDataToConfigs($value, $isFinal);
            if ($isFinal) {
                $this->_wasLicenceSplit = true;
            }
            $this->setValue($splitResult);
            
        }

        return parent::_beforeSave();
    }

    public function save() {
        if (!$this->_isAllowed()) {
            $this->_getEabi()
                    ->getCoreSessionSingleton()
                    ->addError($this->_getEabi()->__('You have no permission to modify licences'));
            
        }
        if (self::$_hasLicenceBeenSplit || !$this->_isAllowed()) {
            return true;
        }
        $res = parent::save();
        if ($this->_wasLicenceSplit) {
            self::$_hasLicenceBeenSplit = true;
        }
        return $res;
    }


    protected function alterLicenceDataToConfigs($rawLicenceValue, &$isNew = false) {
        $paths = explode('/', $this->getPath());
        $currentItem = $paths[1];
        $allMethodCodes = $this->_getEabi()->getAllSupportedMethodCodes();


        $okLicences = array();
        $splitLicences = $this->_getEabi()->getEabiLicenceBlock()->splitLicenceToServices($rawLicenceValue, $isNew);


        foreach ($splitLicences as $methodId => $splitLicence) {

            if ($methodId === $currentItem) {
                $rawLicenceValue = $splitLicence;
                if ($splitLicence) {
                    $okLicences[$currentItem] = $allMethodCodes->getItemByColumnValue('code', $currentItem)->getMethodInstance()->getTitle();
                }
            } else {
                try {
                    $allMethodCodes->getItemByColumnValue('code', $methodId)->setRawLicence($splitLicence);
                    if ($splitLicence) {
                        $okLicences[$methodId] = $allMethodCodes->getItemByColumnValue('code', $methodId)->getMethodInstance()->getTitle();
                    }
                } catch (Exception $ex) {
                    $this->_getEabi()->getEabiLogger()
                            ->setIsLogEnabled(true)
                            ->setLogLevel(Zend_Log::DEBUG)
                            ->error(sprintf('Licence registration failed for method %s', $methodId))
                            ->error($ex);
                }
            }
        }
        if (count($okLicences) && $isNew) {
            $this->_getEabi()
                    ->getCoreSessionSingleton()
                    ->addSuccess($this->_getEabi()->__('Licence OK for following plugins: %s', implode(', ', $okLicences)));
        }
        return $rawLicenceValue;
    }
    
    protected function _isAllowed()  {
        $key = 'eabi_livehandler/eabi_livehandler_licence/request';
        return $this->_getEabi()->getAdminSessionSingleton()
        ->isAllowed($key);
    }
    
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabi() {
        return Mage::helper('eabi');
    }
    

}
