<?php
/*
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Livehandler
 * @copyright  Copyright (c) 2018 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 
 
 */

/**
 * <p>Handles the AJAX request defined for the backend (from database table eabi_livehandler)</p>
 * <p>Security model: If Page itself can be displayed, then it is allowed to run actions, which are bound to current Livehandler model. If user switches page, then current model actions cannot be run any more.</p>
 * <p>If user does not show any activity for Eabi_Livehandler_IndexController::ALLOWED time seconds, then actions cannot be run after timeout.</p>
 *
 * @author matishalmann
 */
class Eabi_Livehandler_Adminhtml_Eabi_Livehandler_LicenceController extends Mage_Adminhtml_Controller_Action {
    protected function _initAction() {
        return $this;
    }
    
    
    
    public function requestAction() {
        //todo: handle licence call request, return json encoded result
        $hosts = $this->_getEabi()
                ->getAllStoreUrls();
        $allLicences = $this->_getLicenceStatus();

        $sealedData = false;
        $keys = array();

        $licenceBlock = $this->_getEabi()->getEabiLicenceBlock();

        $requestData = array(
            'hosts' => $hosts,
            'licences' => $allLicences,
            'return' => Mage::getUrl(),
        );
        openssl_seal(json_encode($requestData), $sealedData, $keys, array($licenceBlock->getPublicKey(true)));

        $data = array(
            'data' => base64_encode($sealedData),
            'key' => base64_encode($keys[0])
        );


        $this->getResponse()
                ->setHeader('Content-type', 'application/json')
                ->setBody(Zend_Json::encode($data));
    }

    protected function _getLicenceStatus() {
        return $this->_getEabi()
                ->getAllLicences();
    }
    
    protected function _isAllowed()  {
        
        $key = 'eabi_livehandler/' . $this->getRequest()->getControllerName() . '/' . $this->getRequest()->getActionName();
        return $this->_getEabi()->getAdminSessionSingleton()
        ->isAllowed($key);
    }
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabi() {
        return Mage::helper('eabi');
    }
    
    
}


