<?php
/*
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Livehandler
 * @copyright  Copyright (c) 2018 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 
 
 */

/**
 * <p>Handles the AJAX request defined for the frontend (from database table eabi_livehandler)</p>
 * <p>Security model: If Page itself can be displayed, then it is allowed to run actions, which are bound to current Livehandler model. If user switches page, then current model actions cannot be run any more.</p>
 * <p>If user does not show any activity for Eabi_Livehandler_IndexController::ALLOWED time seconds, then actions cannot be run after timeout.</p>
 * 
 *
 * @author matishalmann
 */
class Eabi_Livehandler_IndexController extends Mage_Core_Controller_Front_Action {
    const ALLOWED_TIME = 1800;
    public function processAction() {
        $result = array();
        if (!$this->_getEabi()->getConfigData('eabi_livehandler/main/enabled')) {
            throw new Exception('Module Eabi Livehandler is not enabled');
        }
        /*
         * Check if the process is in Session allowed list.
         * 
         */
        $session = Mage::getSingleton('core/session');
        $time = time();
        
        $processName = base64_decode($this->getRequest()->getParam('__path'));
//        $processName = $this->getRequest()->getParam('processName');
        $processEntries = $session->getData('eabi_livehandler_entries');
        $website = Mage::app()->getStore()->getWebsiteId();
        $store = Mage::app()->getStore()->getStoreId();

        
        if (is_array($processEntries) && isset($processEntries[$processName]) && $time - $processEntries[$processName] < self::ALLOWED_TIME) {
            
            
            //execute the action.
            $isAdmin = false;
            $model = $processName;
            
            //get action by action name, website, store, is_admin = false
            $actionsCollection = Mage::getModel('eabi_livehandler/entry')->getCollection()->setModelFilter($model, $isAdmin, $website, $store);
            
            $result = array();
            $classesRan = array();
            foreach ($actionsCollection as $action) {
                $action->load($action->getId());
                if (isset($classesRan[$action->getModelClass()])) {
                    continue;
                }
                
                if (!isset($result['_is_error']) || !$result['_is_error']) {
                    $processEntries[$processName] = $time;
                    $result = $action->runPublic($this->getRequest()->getPost());
                }
                $classesRan[$action->getModelClass()] = true;
                
            }
            
            
            
            Mage::getSingleton('core/session')->setData('eabi_livehandler_entries', $processEntries);
        }
        
        
        echo Zend_Json::encode($result);
        die();
    }
    
    public function getLicenceStatusAction() {
        $allLicences = $this->_getLicenceStatus();
        $licenceBlock = $this->_getEabi()->getEabiLicenceBlock();
        $sealedData = false;
        $keys = array();

        openssl_seal(json_encode($allLicences), $sealedData, $keys, array($licenceBlock->getPublicKey(true)));

        $data = array(
            'data' => base64_encode($sealedData),
            'key' => base64_encode($keys[0])
        );
        $this->getResponse()
                ->setHeader('Content-type', 'application/json')
                ->setBody(Zend_Json::encode($data));
    }

    protected function _getLicenceStatus() {
        return $this->_getEabi()
                ->getAllLicences();
    }
    
    
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabi() {
        return Mage::helper('eabi');
    }
    
}

