<?php
/*

 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 
 
 */

/**
 * <p>Renders yes/no select box with licensing information</p>
 *
 * @author Matis
 */
class Eabi_Postoffice_Block_Adminhtml_Config_Form_Field_License extends Mage_Adminhtml_Block_System_Config_Form_Field {

    protected function _getElementHtml(Varien_Data_Form_Element_Abstract $element) {
        /*
         * structure
         * array()
         * 
         *
         */
        $stores = Mage::app()->getStores();
        $urls = array();
        foreach ($stores as $store) {
            /* @var $store Mage_Core_Model_Store */
            try {
                $url = Zend_Uri_Http::fromString($store->getBaseUrl())->getHost();
                if (!in_array($url, $urls)) {
                    $urls[] = $url;
                }
            } catch (Exception $ex) {
                //do nothing or log
                Mage::log(sprintf('Invalid base url %s for store id %s', $store->getBaseUrl(), $store->getId()), Zend_Log::WARN);

            }
        }
        $date = new Zend_Date(time(), Zend_Date::TIMESTAMP);
        $timezone = Mage::app()->getStore(Mage_Core_Model_App::ADMIN_STORE_ID)->getConfig(Mage_Core_Model_Locale::XML_PATH_DEFAULT_TIMEZONE);
        $date->setTimezone($timezone);
        
        
        $license = array(
            'urls' => $urls,
            'countries' => array(
                'EE' => array('Y', 'Y', 'Y'),
                'LV' => array('T', 'T', 'T'),
                'LT' => array('T', 'N', 'N'),
            ),
            'date' => $date->get(Zend_Date::ISO_8601),
            'sku' => '010125',
            'token' => '12346789',
        );
//        $this->_getOfficeHelper()->setCarrierLicense('eabiomnivaterminal', $license);
        $l = $this->_getOfficeHelper()->getCarrierLicense('eabiomnivaterminal');
//        $license = false;
        $inputData = array(
            "countryName" => Mage::getStoreConfig('general/country/default'), 
            "stateOrProvinceName" => 'N/A',
            "localityName" => 'N/A',
            "organizationName" => 'N/A',
            "organizationalUnitName" => implode(',', $urls),
            "commonName" => 'Aktsiamaailm LLC',
            "emailAddress" => Mage::getStoreConfig('trans_email/ident_general/email'),
        );
        $keys = $this->_getKeypairHelper()->generateKeyPair($inputData);
        
        $text = 'Change license';
        if (!$license) {
            $text = 'Register license';
        }
//        $this->_getEabi()->setConfigData('carriers/eabiomnivaterminal/test', Mage::helper('core')->encrypt('onukala_onukala_onukala_onukala_onukala_onukala_onukala_onukala_onukala'));

        return $element->getElementHtml().'<br/>' . '<pre>'.htmlspecialchars(print_r(Mage::getStoreConfig('carriers/eabiomnivaterminal/test'), true), ENT_COMPAT | ENT_HTML401 | ENT_IGNORE).'</pre>'
            . <<<HTML
                <ul class="eabi_postoffice_license">
                    <li><strong>{$this->_getOfficeHelper()->__('Basic license')}</strong> {$this->_getBasicLicense($license)}</li>
                    <li><strong>{$this->_getOfficeHelper()->__('Automatic data sending')}</strong>{$this->_getDatasendLicense($license)}</li>
                    <li><strong>{$this->_getOfficeHelper()->__('Cash on delivery')}</strong>{$this->_getCodLicense($license)}</li>
                    <li><strong>{$this->_getOfficeHelper()->__('Allowed for URL-s')}</strong>{$this->_getUrlsLicense($license)}</li>
                    <li><strong>&nbsp;</strong><span class="change-license"><a href="">{$this->_getOfficeHelper()->__($text)}</a></span></li>
                </ul>
HTML;
    }
    
    protected function _getUrlsLicense($input) {
        $urls = $input['urls'];
        if (!count($urls)) {
            return '<span class="no-license">'.$this->_getOfficeHelper()->__('not-licensed').'</span>';
        }
        return '<span class="countries">' .implode(', ', $urls).'</span>';
    }
    protected function _getBasicLicense($input) {
        $result = $this->_getGenericLicense($input, 0);
        return $result;
    }
    
    protected function _getDatasendLicense($input) {
        $result = $this->_getGenericLicense($input, 1);
        return $result;
    }
    
    protected function _getCodLicense($input) {
        $result = $this->_getGenericLicense($input, 2);
        return $result;
    }
    
    private function _getGenericLicense($input, $index) {
        $result = array();
        $isTimedOut = false;
        foreach ($input['countries'] as $countryCode => $data) {
            $test = $data[$index];
            if ($test == 'Y') {
                $result[] = $this->_getCountryName($countryCode);
            } else if ($test == 'T' && !$isTimedOut) {
                $result[] = $this->_getCountryName($countryCode) . '-' . $this->_getOfficeHelper()->__('demo');
            }
        }
        if (!count($result)) {
            return '<span class="no-license">'.$this->_getOfficeHelper()->__('not-licensed').'</span>';
        }
        return '<span class="countries">' .implode(', ', $result).'</span>';
    }
    
    private function _getCountryName($code) {
        return Mage::app()->getLocale()->getCountryTranslation($code);
    }


    private function _toJson($input) {
        return json_encode($input);
    }
    
    /**
     * 
     * @return Eabi_Postoffice_Helper_Data
     */
    protected function _getOfficeHelper() {
        return Mage::helper('eabi_postoffice');
    }
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Keypair
     */
    protected function _getKeypairHelper() {
        return Mage::helper('eabi/keypair');
    }
    
    
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabi() {
        return Mage::helper('eabi');
    }

}

