<?php

/*
  
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * <p>Extends Magento Mailer to add attachment functionality and monitor status of mail send action</p>
 *
 * @author Matis
 */
class Eabi_Postoffice_Model_Email_Template_Mailer extends Mage_Core_Model_Email_Template_Mailer {
    
    /**
     * Send all emails from email list
     * @see self::$_emailInfos
     *
     * @return Eabi_Postoffice_Model_Email_Template_Mailer
     */
    public function send()
    {
        $emailTemplate = $this->getEmailTemplate();
        // Send all emails from corresponding list
        while (!empty($this->_emailInfos)) {
            $emailInfo = array_pop($this->_emailInfos);
            // Handle "Bcc" recepients of the current email
            $emailTemplate->addBcc($emailInfo->getBccEmails());
            // Set required design parameters and delegate email sending to Mage_Core_Model_Email_Template
            $emailTemplate->setDesignConfig(array('area' => 'frontend', 'store' => $this->getStoreId()))
                ->sendTransactional(
                $this->getTemplateId(),
                $this->getSender(),
                $emailInfo->getToEmails(),
                $emailInfo->getToNames(),
                $this->getTemplateParams(),
                $this->getStoreId()
            );
        }
        return $this;
    }
    
    /**
     * <p>Resets email sender</p>
     * @return Eabi_Postoffice_Model_Email_Template_Mailer
     */
    public function resetEmailTemplate() {
        $this->unsetData('email_template');
        return $this;
    }

    /**
     * 
     * @return Mage_Core_Model_Email_Template
     */
    public function getEmailTemplate() {
        if ($this->hasData('email_template')) {
            return $this->getData('email_template');
        }
        $this->setData('email_template', $this->_getEmailTemplateModel());
        return $this->getData('email_template');
    }

    /**
     * 
     * @return Mage_Core_Model_Email_Template
     */
    protected function _getEmailTemplateModel() {
        return Mage::getModel('core/email_template');
    }
    
    
}
