<?php

/*
  
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * or OpenGPL v3 license (GNU Public License V3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * or
 * http://www.gnu.org/licenses/gpl-3.0.txt
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@e-abi.ee so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this module to newer
 * versions in the future.
 *
 * @category   Eabi
 * @package    Eabi_Dpd
 * @copyright  Copyright (c) 2016 Aktsiamaailm LLC (http://en.e-abi.ee/)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * @license    http://www.gnu.org/licenses/gpl-3.0.txt  GNU Public License V3.0
 * @author     Matis Halmann
 * 

 */

/**
 * Description of Observer
 *
 * @author Matis
 */
class Eabi_Postoffice_Model_Observer {
    
    
    /**
     * @param Varien_Event_Observer $observer
     */
    public function listenLicense($observer) {
        /* @var $shipmentMethod Eabi_Postoffice_Model_Carrier_Abstract */
        $shipmentMethod = $observer->getEvent()->getShipmentMethod();
        /* @var $request Mage_Shipping_Model_Rate_Request */
        $request = $observer->getEvent()->getRequest();
        $result = $this->_getEabiHelper()
                ->getEabiLicenceBlockSingleton()
                ->isServiceAvailable('shipping/' . $shipmentMethod->getCode(), $request->getDestCountryId());
        $request->setCancelRateCollection(!$result);
    }
    /**
     * @param Varien_Event_Observer $observer
     */
    public function listenAutosendLicense($observer) {
        /* @var $shipmentMethod Eabi_Postoffice_Model_Carrier_Abstract */
        $shipmentMethod = $observer->getEvent()->getShipmentMethod();
        /* @var $request Mage_Shipping_Model_Rate_Request */
        $request = $observer->getEvent()->getRequest();
        $result = $this->_getEabiHelper()
                ->getEabiLicenceBlockSingleton()
                ->isServiceAvailable('autosend/' . $shipmentMethod->getCode(), $request->getDestCountryId());
        $request->setIsCancel(!$result);
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function registerUsage($observer) {
        $order = $observer->getEvent()->getOrder();
        /* @var $order Mage_Sales_Model_Order */
    }
    
    
    /**
     * <p>Validates that pickup point would be selected before order confirmation.</p>
     * <p>Exception is thrown when parcel terminal is not selected</p>
     * @param Varien_Event_Observer $observer
     */
    public function validateSelectedTerminal($observer) {
        /* @var $order Mage_Sales_Model_Order */
        /* @var $quote Mage_Sales_Model_Quote */
        $order = $observer->getEvent()->getOrder();

        $quote = $observer->getEvent()->getQuote();

        if ($order->getIsNotVirtual()) {
            $shippingMethod = $order->getShippingMethod();
            //get the shipping code from the order and call the module from it.
            $shippingCarrierCode = substr($shippingMethod, 0, strpos($shippingMethod, '_'));
            $shippingMethodModel = Mage::getModel('shipping/shipping')->getCarrierByCode($shippingCarrierCode, $order->getStoreId());
            if ($shippingMethodModel && ($shippingMethodModel instanceof Eabi_Postoffice_Model_Carrier_Abstract)) {
                $resultCarrierId = substr($shippingMethod, strrpos($shippingMethod, '_') + 1);
                $result = $this->_getEabiHelper()
                        ->getEabiLicenceBlockSingleton()
                        ->isServiceAvailable('shipping/' . $resultCarrierId, $order->getShippingAddress()->getCountryId());
                if (!$result) {
                    Mage::throwException($this->_getOfficeHelper()->__('Invalid licence for %s', $shippingMethodModel->getConfigData('title')));
                }



                if (!is_numeric($resultCarrierId)) {
                    Mage::throwException($this->_getOfficeHelper()->__('Please select pickup point for %s', $shippingMethodModel->getConfigData('title')));
                }
            }
        }
    }

    /**
     * <p>Prepares email to be sent with new order template and barcode as order comment</p>
     * @param Varien_Event_Observer $observer
     */
    public function dispatchEmailsToCarriers($observer) {
        /* @var $order Mage_Sales_Model_Order */
        /* @var $shipmentMethod Eabi_Postoffice_Model_Carrier_Abstract */
        $order = $observer->getEvent()->getOrder();
        $shipmentMethod = $observer->getEvent()->getShipmentMethod();
        $request = $observer->getEvent()->getRequest();
        $result = $observer->getEvent()->getResult();
        $barcode = $shipmentMethod->getBarcode($order);
        
        if (!$barcode) {
            //no barcode, do not send emails
            return;
        }
        if (is_array($barcode)) {
            $barcode = implode(', ', $barcode);
        }
        
        
        // Start store emulation process
        $appEmulation = Mage::getSingleton('core/app_emulation');
        $initialEnvironmentInfo = $appEmulation->startEnvironmentEmulation($order->getStoreId());

        try {
            // Retrieve specified view block from appropriate design package (depends on emulated store)
            $paymentBlock = Mage::helper('payment')->getInfoBlock($order->getPayment())
                ->setIsSecureMode(true);
            $paymentBlock->getMethod()->setStore($order->getStoreId());
            $paymentBlockHtml = $paymentBlock->toHtml();
        } catch (Exception $exception) {
            // Stop store emulation process
            $appEmulation->stopEnvironmentEmulation($initialEnvironmentInfo);
            throw $exception;
        }
        
        
        // Retrieve corresponding email template id and customer name
        if ($order->getCustomerIsGuest()) {
            $templateId = Mage::getStoreConfig(Mage_Sales_Model_Order::XML_PATH_EMAIL_GUEST_TEMPLATE, $order->getStoreId());
            $customerName = $order->getBillingAddress()->getName();
        } else {
            $templateId = Mage::getStoreConfig(Mage_Sales_Model_Order::XML_PATH_EMAIL_TEMPLATE, $order->getStoreId());
            $customerName = $order->getCustomerName();
        }
        
        
        
        $mailer = $this->_getEmailerModel();
        
        if ($shipmentMethod->getConfigData('enable_log')) {
            Mage::log('auto-send-success-for carrier:' . $shipmentMethod->getCode(), Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
        }
        

        Mage::dispatchEvent('eabi_' . $shipmentMethod->getCode() . '_autosend_data_success', array(
            'request' => $request,
            'shipment_method' => $shipmentMethod,
            'order' => $order,
            'result' => $result,
            /* keep mailer in array, so if would be passed as reference */
            'mailer' => array($mailer),
        ));
        
//        $mailer->addEmailInfo($emailInfo);
        $order->setCustomerNoteNotify(true);
        $order->setCustomerNote($this->_getOfficeHelper()->__('Barcode: %s', $barcode));
        
        // Set all required params and send emails
        $mailer->setSender(Mage::getStoreConfig(Mage_Sales_Model_Order::XML_PATH_EMAIL_IDENTITY, $order->getStoreId()));
        $mailer->setStoreId($order->getStoreId());
        $mailer->setTemplateId($templateId);
        $mailer->setTemplateParams(array(
                'order'        => $order,
                'billing'      => $order->getBillingAddress(),
                'payment_html' => $paymentBlockHtml
            )
        );
        
        if ($shipmentMethod->getConfigData('enable_log')) {
            Mage::log('customer note', Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
            Mage::log(print_r($order->getCustomerNote(), true), Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
        }

        //add the attachment
        if ($barcodePdf = $shipmentMethod->getBarcodePdf($order)) {
            $at = $mailer->getEmailTemplate()->getMail()
                    ->createAttachment($barcodePdf);
            $at->type = 'application/pdf';
            $at->filename = $barcode . '.pdf';
            if ($shipmentMethod->getConfigData('enable_log')) {
                Mage::log('adding attachment', Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
                Mage::log(print_r($at->filename, true), Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
            }
        }

        //send the e-mail
        $mailer->send();
        
        if ($shipmentMethod->getConfigData('enable_log')) {
            Mage::log('mail send result', Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
            Mage::log(print_r($mailer->getEmailTemplate()->getSentSuccess(), true), Zend_Log::DEBUG, get_class($shipmentMethod) . '.log');
        }
        
        
        $mailer->resetEmailTemplate();
        
        
    }
    
    
    public function getAllLicenceAbleShippingMethods($observer) {
        $supportedMethods = $observer->getEvent()->getMethodCodes();
        /* @var $supportedMethods Varien_Data_Collection */

        $allShippingMethods = $this->_getOfficeHelper()
                ->getCarrierModuleModel()
                ->getCollection();
        if (!isset($installedModules['Eabi_Cod']) || !Mage::helper('core')->isModuleEnabled('Eabi_Cod')) {
            //we do not have COD plugin
            Mage::log('E-abi COD plugin is not installed');
        }
        foreach ($allShippingMethods as $shippingMethodEntry) {
            $shippingEntry = $this->_getOfficeHelper()
                    ->getLicenceableShippingMethodModel();
            $shippingEntry->setCode($shippingMethodEntry->getCarrierCode());
            $shippingEntry->setId($shippingEntry->getType() . '/' . $shippingEntry->getCode());
            
            if ($shippingMethodEntry->getNeedsLicense()) {
                $supportedMethods->addItem($shippingEntry);

                //add resulting payment
                $installedModules = (array) Mage::getConfig()->getNode('modules')->children();
                if (isset($installedModules['Eabi_Cod']) && Mage::helper('core')->isModuleEnabled('Eabi_Cod')) {
                    $paymentEntry = $this->_getOfficeHelper()
                            ->getLicenceablePaymentMethodModel();
                    $paymentEntry->setCode($shippingMethodEntry->getCarrierCode());
                    $paymentEntry->setId($paymentEntry->getType() . '/' . $paymentEntry->getCode());

                    $supportedMethods->addItem($paymentEntry);
                }

                //add resulting autosend
                $autosendEntry = $this->_getOfficeHelper()
                        ->getLicenceableAutosendMethodModel();
                $autosendEntry->setCode($shippingMethodEntry->getCarrierCode());
                $autosendEntry->setId($autosendEntry->getType() . '/' . $autosendEntry->getCode());
                
                $supportedMethods->addItem($autosendEntry);
                
                
            }
        }
        
    }
    
    
    

    
    /**
     * 
     * @return Eabi_Postoffice_Model_Email_Template_Mailer
     */
    protected function _getEmailerModel() {
        return Mage::getModel('eabi_postoffice/email_template_mailer');
    }

    /**
     * 
     * @return Eabi_Postoffice_Helper_Data
     */
    protected function _getOfficeHelper() {
        return Mage::helper('eabi_postoffice');
    }
    /**
     * 
     * @return Eabi_Livehandler_Helper_Data
     */
    protected function _getEabiHelper() {
        return Mage::helper('eabi');
    }
    
    

}
